// DirectInput-Hook.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "MyDirectInput.h"
#include "ProxyClassFactory.h"

#ifdef _MANAGED
#pragma managed(push, off)
#endif

void WriteToLog(const char* message);
void WriteToLog(const wchar_t* message);

BOOL APIENTRY DllMain( HMODULE hModule,
					   DWORD  ul_reason_for_call,
					   LPVOID lpReserved
					 )
{
	if (DLL_PROCESS_ATTACH == ul_reason_for_call)
	{
		WriteToLog("Attached To Process:");
		TCHAR process[512];
		GetModuleFileName(GetModuleHandle(0), process, sizeof(process));		
		WriteToLog(process);
	}
	return TRUE;
}

ULONG ProxyClassFactory::sInstanceCount = 0;
ULONG KeyboardDevice::sInstanceCount = 0;
ULONG MyDirectInput::sInstanceCount = 0;

STDAPI DllGetClassObject(const IID& rclsid, const IID& riid, LPVOID* ppv)
{
	WriteToLog("DllGetClassObject called");
	if (riid != IID_IClassFactory)
	{
		WriteToLog("Unexpected IID request");
		return CLASS_E_CLASSNOTAVAILABLE;
	}
	if (ppv == NULL)
	{
		return E_FAIL;
	}
	ProxyClassFactory* factory = new (std::nothrow) ProxyClassFactory();
	if (factory == NULL)
	{
		return E_OUTOFMEMORY;
	}
	factory->AddRef();
	WriteToLog("Factory created");
	*ppv = factory;
	return S_OK;
}

STDAPI DllCanUnloadNow()
{
	ULONG instances = KeyboardDevice::sInstanceCount + MyDirectInput::sInstanceCount + ProxyClassFactory::sInstanceCount;
	return instances > 0 ? S_FALSE : S_OK;
}

STDAPI DllRegisterServer() 
{
	return S_OK;
}

STDAPI DllUnregisterServer() 
{
	return S_OK;
}

#ifdef _DEBUG

#include <fstream>
#include <iostream>
#include <ostream>

void WriteToLog(const char* message)
{
	std::ofstream logfile("c:\\directinputproxy.log", std::ios_base::app);
	logfile << message << std::endl;
}

void WriteToLog(const wchar_t* message)
{
	std::wofstream logfile("c:\\directinputproxy.log", std::ios_base::app);
	logfile << message << std::endl;
}

void WriteGUID(const GUID& guid)
{
	char buffer[300];
	_itoa(guid.Data1, buffer, 16);
	WriteToLog(buffer);
	_itoa(guid.Data2, buffer, 16);
	WriteToLog(buffer);
	_itoa(guid.Data3, buffer, 16);
	WriteToLog(buffer);
}

#else

void WriteToLog(const char* message) {}
void WriteToLog(const wchar_t* message) {}
void WriteGUID(const GUID& guid) {}

#endif


#ifdef _MANAGED
#pragma managed(pop)
#endif