#include "stdafx.h"

#include "MyDirectInput.h"

void WriteToLog(const char* message);
void WriteGUID(const GUID& guid);

HRESULT MyDirectInput::QueryInterface(REFIID riid, void** ppvObj)
{
	WriteToLog("QueryInterface called, iid:");
	WriteGUID(riid);
	if (riid == IID_IDirectInput7W)
	{
		if (ppvObj == NULL)
		{
			return E_POINTER;
		}
		*ppvObj = this;
		AddRef();
		return S_OK;
	}
	else
	{
		return m_di->QueryInterface(riid, ppvObj);
	}
}

IDirectInputDeviceW* CreateKeyboardProxy(IDirectInputDevice7* realDevice)
{
		TCHAR targetProcess[512];
		DWORD count = 512;

		HKEY appKey = 0;
		if(ERROR_SUCCESS == RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\DirectInputProxy"), 0, KEY_QUERY_VALUE,
			&appKey))
		{
			if(ERROR_SUCCESS == RegQueryValueEx(appKey, 0, 0, 0, reinterpret_cast<BYTE*>(targetProcess), &count))
			{
				TCHAR process[512];
				GetModuleFileName(GetModuleHandle(0), process, 512);
				PathStripPath(process);
				
				if(_tcsnicmp(targetProcess, process, 512) == 0)
				{
					WriteToLog("Creating keyboard proxy");
					return new KeyboardDevice(realDevice);	
				}
			}
			else
			{
				WriteToLog("Couldn't read registry key.");
			}

			RegCloseKey(appKey);
		}
		else
		{
			WriteToLog("Couldn't open registry key.");
		}
		return realDevice;
}


HRESULT MyDirectInput::CreateDevice(const GUID &rguid, IDirectInputDeviceW **device, LPUNKNOWN unknown)
{
	HRESULT hr = m_di->CreateDevice(rguid, device, unknown);
	if(SUCCEEDED(hr) && rguid == GUID_SysKeyboard)
	{

		WriteToLog("Requesting keyboard");
		// Return our own keyboard device that checks for injected keypresses
		*device = CreateKeyboardProxy(static_cast<IDirectInputDevice7*>(*device));
	}
	else if SUCCEEDED(hr)
	{
		WriteToLog("Creating other device:");
		WriteGUID(rguid);
	}

	return hr;
}

HRESULT MyDirectInput::CreateDeviceEx(REFGUID rguid, REFIID riid, LPVOID * device, LPUNKNOWN unknown)
{
	HRESULT hr = m_di->CreateDeviceEx(rguid, riid, device, unknown);
	if (SUCCEEDED(hr) && rguid == GUID_SysKeyboard)
	{
		WriteToLog("Requesting keyboard");
		*device = CreateKeyboardProxy(static_cast<IDirectInputDevice7*>(*device));
	}
	else if SUCCEEDED(hr)
	{
		WriteToLog("Creating other device:");
		WriteGUID(rguid);
	}

	return hr;
}

HRESULT MyDirectInput::FindDevice(const GUID& rguid, LPCWSTR str, LPGUID pGuid)
{
	WriteToLog("FindDevice called:");
	WriteGUID(rguid);

	HRESULT hr = m_di->FindDevice(rguid, str, pGuid);

	if (SUCCEEDED(hr))
	{
		WriteToLog("Succeeed, guid is: ");
		WriteGUID(*pGuid);
	}
	return hr;
}

HRESULT MyDirectInput::EnumDevices(DWORD devType,LPDIENUMDEVICESCALLBACK callback, LPVOID ref, DWORD flags)
{
	WriteToLog("EnumDevices called.");
	return m_di->EnumDevices(devType, callback, ref, flags);
}
