#include "stdafx.h"
#include "ProxyClassFactory.h"
#include "MyDirectInput.h"

#include <dinput.h>

void WriteToLog(const char* message);
void WriteGUID(const GUID& guid);

class DInputWrapper
{
private:
	typedef HRESULT (WINAPI *DirectInputCreateFct)(HINSTANCE hinst, DWORD dwVersion, LPDIRECTINPUTW *ppDI, LPUNKNOWN punkOuter);
	DirectInputCreateFct mFct;
	HMODULE mModule;

	DInputWrapper()
		: mFct(NULL), mModule(NULL)
	{
		mModule = ::LoadLibrary(_T("dinput.dll"));
		if (mModule == NULL)
		{
			WriteToLog("Couldn't load dinput.dll");
			return;
		}
		mFct = (DirectInputCreateFct)::GetProcAddress(mModule, "DirectInputCreateW");
		if (mFct == NULL)
		{
			WriteToLog("Couldn't find DirectInputCreateW");
			return;
		}
	}

	~DInputWrapper()
	{
		if (mModule) ::FreeLibrary(mModule);
	}
public:

	static DInputWrapper& Instance()
	{
		static DInputWrapper sInstance;
		return sInstance;
	}

	HRESULT CreateDInput(HINSTANCE module, DWORD version, IDirectInput** ppObj)
	{
		if (mFct)
		{
			return mFct(module, version, ppObj, NULL);
		}
		else
		{
			return E_FAIL;
		}
	}
};

HRESULT ProxyClassFactory::CreateInstance(IUnknown *pUnkOuter, const IID &riid, void **ppvObject)
{
	if (riid != IID_IDirectInput7)
	{
		WriteToLog("Wrong IID requested from factory:");
		WriteGUID(riid);
		return E_UNEXPECTED;
	}
	if (ppvObject == NULL)
	{
		return E_FAIL;
	}
	IDirectInput* realObject = NULL;
	HRESULT hr = DInputWrapper::Instance().CreateDInput(::GetModuleHandle(_T("DirectInputProxy")), DIRECTINPUT_VERSION, &realObject);
	if (hr != DI_OK)
	{
		WriteToLog("Couldn't create real object");
		return E_UNEXPECTED;
	}
	MyDirectInput* proxyObject = new(std::nothrow)MyDirectInput(static_cast<IDirectInput7*>(realObject));
	if (proxyObject == NULL)
	{
		WriteToLog("Out of memory");
		return E_OUTOFMEMORY;
	}
	WriteToLog("Proxy object created OK");
	*ppvObject = proxyObject;
	return S_OK;
}