﻿using System;
using System.Collections.Generic;
using System.Text;

namespace RPGSoundController
{
    enum MessageType
    {
        Debug = 0, Info = 1, Warning = 2, Error = 3
    }

    struct Message
    {
        public MessageType Type { get; set; }
        public String Text { get; set; }

        public Message(MessageType type, String text)
            : this()
        {
            Type = type;
            Text = text;
        }
    }

    delegate void MessageReceivedHandler(Message m);

    sealed class Messages
    {
        public static Messages Instance
        {
            get
            {
                if (sInstance == null)
                {
                    sInstance = new Messages();
                }
                return sInstance;
            }
        }

        private static Messages sInstance = null;

        private List<Message> mMessages = null;

        public IList<Message> GetAllMessages() { lock (this) { return mMessages; } }

        private Messages()
        {
            mMessages = new List<Message>();
        }

        public event MessageReceivedHandler MessageReceived;

        private void AddMessage(Message m)
        {
            lock (this)
            {
                mMessages.Add(m);
            }
            lock (Options.Default)
            {
                if (MessageReceived != null && (int)m.Type >= Options.Default.MessageFilter)
                {
                    MessageReceived(m);
                }
            }
        }

        public static void AddMessage(MessageType type, String text)
        {
            Instance.AddMessage(new Message(type, text));
        }
    }
}
