﻿using System;


namespace RPGSoundController
{
    class Player
    {
        public Player()
        {
            Connected = false;
        }

        public bool InitConnection()
        {
            dInput = new DirectInputConnect();
            System.IO.FileInfo info = new System.IO.FileInfo(Options.Default.PlayerPath);
            if (!info.Exists)
            {
                Messages.AddMessage(MessageType.Error, Resources.PlayerNotFound);
                return false;
            }
            if (dInput.SetTargetExecutable(info.Name) != 0)
            {
                Messages.AddMessage(MessageType.Error, Resources.TargetForProxyNotSet);
                return false;
            }
            if (dInput.InstallProxy() != 0)
            {
                Messages.AddMessage(MessageType.Error, Resources.ProxyNotInstalled);
                return false;
            }
            System.Diagnostics.ProcessStartInfo startInfo = new System.Diagnostics.ProcessStartInfo(Options.Default.PlayerPath);
            startInfo.WorkingDirectory = info.Directory.FullName;
            playerProcess = System.Diagnostics.Process.Start(startInfo);
            int time = 0;
            while (!playerProcess.Responding && time < 10000)
            {
                System.Threading.Thread.Sleep(50);
                time += 50;
            }
            if (!playerProcess.Responding)
            {
                Messages.AddMessage(MessageType.Error, Resources.PlayerNotStarted);
                return false;
            }
            if (dInput.Connect(3000) != 0)
            {
                dInput.UnnstallProxy();
                dInput.Dispose();
                dInput = null;
                Messages.AddMessage(MessageType.Error, Resources.ProxyNotConnected);
                return false;
            }
            Messages.AddMessage(MessageType.Info, Resources.PlayerConnected);
            Connected = true;
            return true;
        }

        public void CloseConnection()
        {
            if (!Connected) return;
            Connected = false;
            dInput.Disconnect();
            dInput.UnnstallProxy();
            dInput.Dispose();
            dInput = null;
            playerProcess.CloseMainWindow();
            Messages.AddMessage(MessageType.Info, "");
        }

        public void SendKeyToPlayer(System.Windows.Forms.Keys keyCode)
        {
            if (Connected)
            {
                Messages.AddMessage(MessageType.Debug, String.Format(Resources.KeySending, keyCode));
                dInput.Send(keyCode);
            }
            else
            {
                Messages.AddMessage(MessageType.Warning, Resources.NoConnectionNoSend);
            }
        }

        public bool Connected { get; set; }

        private DirectInputConnect dInput = null;
        private System.Diagnostics.Process playerProcess;
    }
}
